package jp.gmail.tdr;

import java.util.ArrayList;
import java.util.Deque;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.aiwolf.client.lib.ComingoutContentBuilder;
import org.aiwolf.client.lib.Content;
import org.aiwolf.client.lib.DivinedResultContentBuilder;
import org.aiwolf.client.lib.EstimateContentBuilder;
import org.aiwolf.client.lib.RequestContentBuilder;
import org.aiwolf.client.lib.VoteContentBuilder;
import org.aiwolf.common.data.Agent;
import org.aiwolf.common.data.Judge;
import org.aiwolf.common.data.Role;
import org.aiwolf.common.data.Species;
import org.aiwolf.common.net.GameInfo;
import org.aiwolf.common.net.GameSetting;

public class Myseer extends Myvillager {
	int comingoutDay;
	boolean isCameout;
	Deque<Judge> divinationQueue = new LinkedList<>();
	Map<Agent, Species> myDivinationMap = new HashMap<>();
	List<Agent> whiteList = new ArrayList<>();
	List<Agent> blackList = new ArrayList<>();
	List<Agent> grayList;
	List<Agent> possessedList = new ArrayList<>();
	Agent toBeDivined;

	@Override
	protected void chooseVoteCandidate() {
		boolean fixed = false;

		List<Agent> aliveWolves = new ArrayList<>();
		for (Agent a : blackList) {
			if (isAlive(a)) {
				aliveWolves.add(a);
			}
		}

		if (!aliveWolves.isEmpty()) {
			if (!aliveWolves.contains(voteCandidate)) {
				voteCandidate = randomSelect(aliveWolves);
				fixed = true;
				if (canTalk) {
					talkQueue
							.offer(new Content(new RequestContentBuilder(null,
									new Content(new VoteContentBuilder(
											voteCandidate)))));
				}
			}
		}

		werewolves.clear();

		for (Agent a : aliveOthers) {
			if (comingoutMap.get(a) == Role.SEER) {
				if (whiteList.contains(a)) {
					if (!possessedList.contains(a)) {
						possessedList.add(a);
						talkQueue.offer(new Content(new EstimateContentBuilder(
								a, Role.POSSESSED)));
					}
				} else {
					werewolves.add(a);
				}
			}
		}

		toBeDivined = randomSelect(werewolves);
		werewolves.remove(toBeDivined);

		for (Judge j : identList) {
			Agent agent = j.getAgent();
			if ((myDivinationMap.containsKey(j.getTarget()) && j.getResult() != myDivinationMap
					.get(j.getTarget()))) {
				if (isAlive(agent) && !werewolves.contains(agent)) {
					if (whiteList.contains(agent)) {
						if (!possessedList.contains(agent)) {
							possessedList.add(agent);
							talkQueue.offer(new Content(
									new EstimateContentBuilder(agent,
											Role.POSSESSED)));
						}
					} else {
						werewolves.add(agent);
					}
				}
			}
		}
		if (!fixed) {
			if (!werewolves.isEmpty()) {
				if (!werewolves.contains(voteCandidate)) {
					voteCandidate = randomSelect(werewolves);

					if (canTalk) {
						talkQueue.offer(new Content(new EstimateContentBuilder(
								voteCandidate, Role.WEREWOLF)));
					}
				}
			}

			else {
				if (!grayList.isEmpty()) {
					if (!grayList.contains(voteCandidate)) {
						voteCandidate = randomSelect(grayList);
					}
				}

				else {
					if (!aliveOthers.contains(voteCandidate)) {
						voteCandidate = randomSelect(aliveOthers);
					}
				}
			}
		}
	}

	@Override
	public void initialize(GameInfo gameInfo, GameSetting gameSetting) {
		super.initialize(gameInfo, gameSetting);
		comingoutDay = (int) (Math.random() * 3 + 1);
		isCameout = false;
		divinationQueue.clear();
		myDivinationMap.clear();
		whiteList.clear();
		blackList.clear();
		grayList = new ArrayList<>(gameInfo.getAliveAgentList());
		grayList.remove(me);
		possessedList.clear();
	}

	@Override
	public void dayStart() {
		super.dayStart();
		toBeDivined = null;

		Judge divination = currentGameInfo.getDivineResult();
		if (divination != null) {
			divinationQueue.offer(divination);
			grayList.remove(divination.getTarget());
			if (divination.getResult() == Species.HUMAN) {
				whiteList.add(divination.getTarget());
			} else {
				blackList.add(divination.getTarget());
			}
			myDivinationMap.put(divination.getTarget(), divination.getResult());
		}
	}

	@Override
	public String talk() {

		if (!isCameout
				&& (day >= comingoutDay
						|| (!divinationQueue.isEmpty() && divinationQueue
								.peek().getResult() == Species.WEREWOLF) || isCo(Role.SEER))) {
			talkQueue.offer(new Content(new ComingoutContentBuilder(me,
					Role.SEER)));
			isCameout = true;
		}

		if (isCameout) {
			while (!divinationQueue.isEmpty()) {
				Judge ident = divinationQueue.poll();
				talkQueue.offer(new Content(new DivinedResultContentBuilder(
						ident.getTarget(), ident.getResult())));
			}
		}
		return super.talk();
	}

	@Override
	public Agent divine() {
		if (toBeDivined != null) {
			Agent candidate = toBeDivined;
			toBeDivined = null;
			return candidate;
		}

		if (!werewolves.isEmpty()) {
			return randomSelect(werewolves);
		}

		List<Agent> candidates = new ArrayList<>();
		for (Agent a : aliveOthers) {
			if (!myDivinationMap.containsKey(a)) {
				candidates.add(a);
			}
		}
		if (candidates.isEmpty()) {
			return null;
		}
		return randomSelect(candidates);
	}
}
